NEWLINE = '\n'

from pygame_phyics.objects.ui.ui import UI
from pygame_phyics.objects.ui.button import Button
from pygame_phyics.objects.ui.inputfield import InputField
from pygame_phyics.objects.ui.text import Text