
__version__ = "1.1.2"

PPM =20

from pygame_phyics.manger import Manger
from pygame_phyics.game import Game
from pygame_phyics.objects import *
from pygame_phyics.sheet import SurfaceSheet
from pygame_phyics.vector import Vector
from pygame_phyics.timertask import TimerTask, OnceTimerTask
from pygame_phyics.input import Input
from pygame_phyics.animation import Animation

from pygame import (
    Surface,
    Rect
)

from pygame.constants import *
